@@ -1,5 +1,5 @@
 /**********************************************************************
-  iso8859_10.c -  Oniguruma (regular expression library)
+  iso8859_13.c -  Oniguruma (regular expression library)
 **********************************************************************/
 /*-
  * Copyright (c) 2002-2007  K.Kosako  <sndgk393 AT ybb DOT ne DOT jp>
@@ -30,11 +30,11 @@
 #include "regenc.h"
 #include "iso_8859.h"
 
-#define ENC_ISO_8859_10_TO_LOWER_CASE(c) EncISO_8859_10_ToLowerCaseTable[c]
-#define ENC_IS_ISO_8859_10_CTYPE(code,ctype) \
-  ((EncISO_8859_10_CtypeTable[code] & CTYPE_TO_BIT(ctype)) != 0)
+#define ENC_ISO_8859_13_TO_LOWER_CASE(c) EncISO_8859_13_ToLowerCaseTable[c]
+#define ENC_IS_ISO_8859_13_CTYPE(code,ctype) \
+  ((EncISO_8859_13_CtypeTable[code] & CTYPE_TO_BIT(ctype)) != 0)
 
-static const UChar EncISO_8859_10_ToLowerCaseTable[256] = {
+static const UChar EncISO_8859_13_ToLowerCaseTable[256] = {
   '\000', '\001', '\002', '\003', '\004', '\005', '\006', '\007',
   '\010', '\011', '\012', '\013', '\014', '\015', '\016', '\017',
   '\020', '\021', '\022', '\023', '\024', '\025', '\026', '\027',
@@ -55,13 +55,13 @@ static const UChar EncISO_8859_10_ToLowerCaseTable[256] = {
   '\210', '\211', '\212', '\213', '\214', '\215', '\216', '\217',
   '\220', '\221', '\222', '\223', '\224', '\225', '\226', '\227',
   '\230', '\231', '\232', '\233', '\234', '\235', '\236', '\237',
-  '\240', '\261', '\262', '\263', '\264', '\265', '\266', '\247',
-  '\270', '\271', '\272', '\273', '\274', '\255', '\276', '\277',
+  '\240', '\241', '\242', '\243', '\244', '\245', '\246', '\247',
+  '\270', '\251', '\272', '\253', '\254', '\255', '\256', '\277',
   '\260', '\261', '\262', '\263', '\264', '\265', '\266', '\267',
   '\270', '\271', '\272', '\273', '\274', '\275', '\276', '\277',
   '\340', '\341', '\342', '\343', '\344', '\345', '\346', '\347',
   '\350', '\351', '\352', '\353', '\354', '\355', '\356', '\357',
-  '\360', '\361', '\362', '\363', '\364', '\365', '\366', '\367',
+  '\360', '\361', '\362', '\363', '\364', '\365', '\366', '\327',
   '\370', '\371', '\372', '\373', '\374', '\375', '\376', '\337',
   '\340', '\341', '\342', '\343', '\344', '\345', '\346', '\347',
   '\350', '\351', '\352', '\353', '\354', '\355', '\356', '\357',
@@ -69,7 +69,7 @@ static const UChar EncISO_8859_10_ToLowerCaseTable[256] = {
   '\370', '\371', '\372', '\373', '\374', '\375', '\376', '\377'
 };
 
-static const unsigned short EncISO_8859_10_CtypeTable[256] = {
+static const unsigned short EncISO_8859_13_CtypeTable[256] = {
   0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008,
   0x4008, 0x420c, 0x4209, 0x4208, 0x4208, 0x4208, 0x4008, 0x4008,
   0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008,
@@ -90,18 +90,18 @@ static const unsigned short EncISO_8859_10_CtypeTable[256] = {
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
-  0x0284, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x00a0,
-  0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x01a0, 0x34a2, 0x34a2,
-  0x00a0, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x01a0,
-  0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x01a0, 0x30e2, 0x30e2,
-  0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2,
+  0x0284, 0x01a0, 0x00a0, 0x00a0, 0x00a0, 0x01a0, 0x00a0, 0x00a0,
+  0x34a2, 0x00a0, 0x34a2, 0x01a0, 0x00a0, 0x01a0, 0x00a0, 0x34a2,
+  0x00a0, 0x00a0, 0x10a0, 0x10a0, 0x01a0, 0x30e2, 0x00a0, 0x01a0,
+  0x30e2, 0x10a0, 0x30e2, 0x01a0, 0x10a0, 0x10a0, 0x10a0, 0x30e2,
   0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2,
   0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2,
+  0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x00a0,
   0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x30e2,
   0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2,
   0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2,
-  0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2,
-  0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2
+  0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x00a0,
+  0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x01a0
 };
 
 static int
@@ -118,7 +118,7 @@ mbc_case_fold(OnigCaseFoldType flag,
     return 2;
   }
 
-  *lower = ENC_ISO_8859_10_TO_LOWER_CASE(*p);
+  *lower = ENC_ISO_8859_13_TO_LOWER_CASE(*p);
   (*pp)++;
   return 1;
 }
@@ -136,9 +136,13 @@ is_mbc_ambiguous(OnigCaseFoldType flag, const UChar** pp, const UChar* end)
   }
 
   (*pp)++;
-  v = (EncISO_8859_10_CtypeTable[*p] & (BIT_CTYPE_UPPER | BIT_CTYPE_LOWER));
+  v = (EncISO_8859_13_CtypeTable[*p] & (BIT_CTYPE_UPPER | BIT_CTYPE_LOWER));
   if ((v | BIT_CTYPE_LOWER) != 0) {
-    return TRUE;
+    /* 0xdf, 0xb5 are lower case letter, but can't convert. */
+    if (*p == 0xb5)
+      return FALSE;
+    else
+      return TRUE;
   }
 
   return (v != 0 ? TRUE : FALSE);
@@ -149,24 +153,14 @@ static int
 is_code_ctype(OnigCodePoint code, unsigned int ctype, OnigEncoding enc ARG_UNUSED)
 {
   if (code < 256)
-    return ENC_IS_ISO_8859_10_CTYPE(code, ctype);
+    return ENC_IS_ISO_8859_13_CTYPE(code, ctype);
   else
     return FALSE;
 }
 
 static const OnigPairCaseFoldCodes CaseFoldMap[] = {
- { 0xa1, 0xb1 },
- { 0xa2, 0xb2 },
- { 0xa3, 0xb3 },
- { 0xa4, 0xb4 },
- { 0xa5, 0xb5 },
- { 0xa6, 0xb6 },
  { 0xa8, 0xb8 },
- { 0xa9, 0xb9 },
  { 0xaa, 0xba },
- { 0xab, 0xbb },
- { 0xac, 0xbc },
- { 0xae, 0xbe },
  { 0xaf, 0xbf },
 
  { 0xc0, 0xe0 },
@@ -193,7 +187,6 @@ static const OnigPairCaseFoldCodes CaseFoldMap[] = {
  { 0xd4, 0xf4 },
  { 0xd5, 0xf5 },
  { 0xd6, 0xf6 },
- { 0xd7, 0xf7 },
  { 0xd8, 0xf8 },
  { 0xd9, 0xf9 },
  { 0xda, 0xfa },
@@ -205,8 +198,8 @@ static const OnigPairCaseFoldCodes CaseFoldMap[] = {
 
 static int
 apply_all_case_fold(OnigCaseFoldType flag,
-			       OnigApplyAllCaseFoldFunc f, void* arg,
-			       OnigEncoding enc ARG_UNUSED)
+				OnigApplyAllCaseFoldFunc f, void* arg,
+				OnigEncoding enc ARG_UNUSED)
 {
   return onigenc_apply_all_case_fold_with_map(
              numberof(CaseFoldMap), CaseFoldMap, 1,
@@ -247,20 +240,22 @@ case_map(OnigCaseFoldType* flagP, const OnigUChar** pp,
 	code = 's';
       }
     }
-    else if (code == 0xBD || code == 0xFF)
-      ;
-    else if ((EncISO_8859_10_CtypeTable[code] & BIT_CTYPE_UPPER)
+    else if ((EncISO_8859_13_CtypeTable[code] & BIT_CTYPE_UPPER)
 	     && (flags & (ONIGENC_CASE_DOWNCASE | ONIGENC_CASE_FOLD))) {
       flags |= ONIGENC_CASE_MODIFIED;
-      code = ENC_ISO_8859_10_TO_LOWER_CASE(code);
+      code = ENC_ISO_8859_13_TO_LOWER_CASE(code);
     }
-    else if ((EncISO_8859_10_CtypeTable[code] & BIT_CTYPE_LOWER)
+    else if (code == 0xB5)
+      ;
+    else if ((EncISO_8859_13_CtypeTable[code] & BIT_CTYPE_LOWER)
 	     && (flags & ONIGENC_CASE_UPCASE)) {
       flags |= ONIGENC_CASE_MODIFIED;
-      if (code >= 0xA0 && code <= 0xBF)
-	code -= 0x10;
-      else
-	code -= 0x20;
+      if (code == 0xB8 || code == 0xBA || code == 0xBF) {
+        code -= 0x10;
+      }
+      else {
+        code -= 0x20;
+      }
     }
     *to++ = code;
     if (flags & ONIGENC_CASE_TITLECASE)  /* switch from titlecase to lowercase for capitalize */
@@ -270,9 +265,9 @@ case_map(OnigCaseFoldType* flagP, const OnigUChar** pp,
   return (int )(to - to_start);
 }
 
-OnigEncodingDefine(iso_8859_10, ISO_8859_10) = {
+OnigEncodingDefine(iso_8859_13, ISO_8859_13) = {
   onigenc_single_byte_mbc_enc_len,
-  "ISO-8859-10", /* name */
+  "ISO-8859-13",  /* name */
   1,             /* max enc length */
   1,             /* min enc length */
   onigenc_is_mbc_newline_0x0a,
@@ -291,4 +286,4 @@ OnigEncodingDefine(iso_8859_10, ISO_8859_10) = {
   0,
   ONIGENC_FLAG_NONE,
 };
-ENC_ALIAS("ISO8859-10", "ISO-8859-10")
+ENC_ALIAS("ISO8859-13", "ISO-8859-13")
