@@ -31,7 +31,7 @@ describe :array_pack_basic_non_float, shared: true do
     # 2 additional directives ('a') are required for the X directive
     [@obj, @obj, @obj, @obj].pack("aa #{pack_format} # some comment \n#{pack_format}").should be_an_instance_of(String)
   end
-  
+
   ruby_version_is ""..."3.2" do
     it "warns in verbose mode that a directive is unknown" do
       # additional directive ('a') is required for the X directive
@@ -57,9 +57,9 @@ describe :array_pack_basic_non_float, shared: true do
     # NOTE: Added this case just to not forget about the decision in the ticket
     it "raise ArgumentError when a directive is unknown" do
       # additional directive ('a') is required for the X directive
-      -> { [@obj, @obj].pack("a R" + pack_format) }.should raise_error(ArgumentError)
-      -> { [@obj, @obj].pack("a 0" + pack_format) }.should raise_error(ArgumentError)
-      -> { [@obj, @obj].pack("a :" + pack_format) }.should raise_error(ArgumentError)
+      -> { [@obj, @obj].pack("a R" + pack_format) }.should raise_error(ArgumentError, /unknown pack directive 'R'/)
+      -> { [@obj, @obj].pack("a 0" + pack_format) }.should raise_error(ArgumentError, /unknown pack directive '0'/)
+      -> { [@obj, @obj].pack("a :" + pack_format) }.should raise_error(ArgumentError, /unknown pack directive ':'/)
     end
   end
 
