@@ -263,18 +263,21 @@ describe "A lambda literal -> () { }" do
     end
 
     describe "with circular optional argument reference" do
-      it "raises a SyntaxError if using an existing local with the same name as the argument" do
-        a = 1
-        -> {
-          @proc = eval "-> (a=a) { a }"
-        }.should raise_error(SyntaxError)
+      ruby_version_is ""..."3.4" do
+        it "raises a SyntaxError if using the argument in its default value" do
+          a = 1
+          -> {
+            eval "-> (a=a) { a }"
+          }.should raise_error(SyntaxError)
+        end
       end
 
-      it "raises a SyntaxError if there is an existing method with the same name as the argument" do
-        def a; 1; end
-        -> {
-          @proc = eval "-> (a=a) { a }"
-        }.should raise_error(SyntaxError)
+      ruby_version_is "3.4" do
+        it "is nil if using the argument in its default value" do
+          -> {
+            eval "-> (a=a) { a }.call"
+          }.call.should == nil
+        end
       end
 
       it "calls an existing method with the same name as the argument if explicitly using ()" do
