@@ -167,9 +167,10 @@ module TestIRB
       orig_use_autocomplete_env = ENV['IRB_COMPLETOR']
       orig_use_autocomplete_conf = IRB.conf[:COMPLETOR]
 
+      # Default value is nil: auto-detect
       ENV['IRB_COMPLETOR'] = nil
       IRB.setup(__FILE__)
-      assert_equal(:regexp, IRB.conf[:COMPLETOR])
+      assert_equal(nil, IRB.conf[:COMPLETOR])
 
       ENV['IRB_COMPLETOR'] = 'regexp'
       IRB.setup(__FILE__)
@@ -193,10 +194,12 @@ module TestIRB
 
     def test_completor_setup_with_argv
       orig_completor_conf = IRB.conf[:COMPLETOR]
+      orig_completor_env = ENV['IRB_COMPLETOR']
+      ENV['IRB_COMPLETOR'] = nil
 
-      # Default is :regexp
+      # Default value is nil: auto-detect
       IRB.setup(__FILE__, argv: [])
-      assert_equal :regexp, IRB.conf[:COMPLETOR]
+      assert_equal nil, IRB.conf[:COMPLETOR]
 
       IRB.setup(__FILE__, argv: ['--type-completor'])
       assert_equal :type, IRB.conf[:COMPLETOR]
@@ -205,6 +208,7 @@ module TestIRB
       assert_equal :regexp, IRB.conf[:COMPLETOR]
     ensure
       IRB.conf[:COMPLETOR] = orig_completor_conf
+      ENV['IRB_COMPLETOR'] = orig_completor_env
     end
 
     def test_noscript
@@ -268,15 +272,93 @@ module TestIRB
     end
   end
 
+  class ConfigValidationTest < TestCase
+    def setup
+      # To prevent the test from using the user's .irbrc file
+      @home = Dir.mktmpdir
+      setup_envs(home: @home)
+      super
+    end
+
+    def teardown
+      super
+      teardown_envs
+      File.unlink(@irbrc)
+      Dir.rmdir(@home)
+      IRB.instance_variable_set(:@existing_rc_name_generators, nil)
+    end
+
+    def test_irb_name_converts_non_string_values_to_string
+      assert_no_irb_validation_error(<<~'RUBY')
+        IRB.conf[:IRB_NAME] = :foo
+      RUBY
+
+      assert_equal "foo", IRB.conf[:IRB_NAME]
+    end
+
+    def test_irb_rc_name_only_takes_callable_objects
+      assert_irb_validation_error(<<~'RUBY', "IRB.conf[:IRB_RC] should be a callable object. Got :foo.")
+        IRB.conf[:IRB_RC] = :foo
+      RUBY
+    end
+
+    def test_back_trace_limit_only_accepts_integers
+      assert_irb_validation_error(<<~'RUBY', "IRB.conf[:BACK_TRACE_LIMIT] should be an integer. Got \"foo\".")
+        IRB.conf[:BACK_TRACE_LIMIT] = "foo"
+      RUBY
+    end
+
+    def test_prompt_only_accepts_hash
+      assert_irb_validation_error(<<~'RUBY', "IRB.conf[:PROMPT] should be a Hash. Got \"foo\".")
+        IRB.conf[:PROMPT] = "foo"
+      RUBY
+    end
+
+    def test_eval_history_only_accepts_integers
+      assert_irb_validation_error(<<~'RUBY', "IRB.conf[:EVAL_HISTORY] should be an integer. Got \"foo\".")
+        IRB.conf[:EVAL_HISTORY] = "foo"
+      RUBY
+    end
+
+    private
+
+    def assert_irb_validation_error(rc_content, error_message)
+      write_rc rc_content
+
+      assert_raise_with_message(TypeError, error_message) do
+        IRB.setup(__FILE__)
+      end
+    end
+
+    def assert_no_irb_validation_error(rc_content)
+      write_rc rc_content
+
+      assert_nothing_raised do
+        IRB.setup(__FILE__)
+      end
+    end
+
+    def write_rc(content)
+      @irbrc = Tempfile.new('irbrc')
+      @irbrc.write(content)
+      @irbrc.close
+      ENV['IRBRC'] = @irbrc.path
+    end
+  end
+
   class InitIntegrationTest < IntegrationTestCase
-    def test_load_error_in_rc_file_is_warned
-      write_rc <<~'IRBRC'
-        require "file_that_does_not_exist"
-      IRBRC
+    def setup
+      super
 
       write_ruby <<~'RUBY'
         binding.irb
       RUBY
+    end
+
+    def test_load_error_in_rc_file_is_warned
+      write_rc <<~'IRBRC'
+        require "file_that_does_not_exist"
+      IRBRC
 
       output = run_ruby_file do
         type "'foobar'"
@@ -293,10 +375,6 @@ module TestIRB
         raise "I'm an error"
       IRBRC
 
-      write_ruby <<~'RUBY'
-        binding.irb
-      RUBY
-
       output = run_ruby_file do
         type "'foobar'"
         type "exit"
