@@ -840,30 +840,6 @@ class TestRequire < Test::Unit::TestCase
         p :ok
       end;
     }
-
-    # [Bug #21567]
-    assert_separately(%w[-rtempfile], "#{<<~"begin;"}\n#{<<~"end;"}")
-    begin;
-      class MyString
-        def initialize(path)
-          @path = path
-        end
-
-        def to_str
-          $LOADED_FEATURES.clear
-          @path
-        end
-
-        def to_path = @path
-      end
-
-      def create_ruby_file = Tempfile.create(["test", ".rb"]).path
-
-      require MyString.new(create_ruby_file)
-      $LOADED_FEATURES.unshift(create_ruby_file)
-      $LOADED_FEATURES << MyString.new(create_ruby_file)
-      require create_ruby_file
-    end;
   end
 
   def test_loading_fifo_threading_raise
@@ -871,7 +847,7 @@ class TestRequire < Test::Unit::TestCase
       f.close
       File.unlink(f.path)
       File.mkfifo(f.path)
-      assert_separately(["-", f.path], "#{<<~"begin;"}\n#{<<~"end;"}", timeout: 3)
+      assert_separately(["-", f.path], "#{<<~"begin;"}\n#{<<~"end;"}", timeout: 10)
       begin;
         th = Thread.current
         Thread.start {begin sleep(0.001) end until th.stop?; th.raise(IOError)}
@@ -890,7 +866,7 @@ class TestRequire < Test::Unit::TestCase
       File.unlink(f.path)
       File.mkfifo(f.path)
 
-      assert_separately(["-", f.path], "#{<<~"begin;"}\n#{<<~"end;"}", timeout: 3)
+      assert_separately(["-", f.path], "#{<<~"begin;"}\n#{<<~"end;"}", timeout: 10)
       begin;
         path = ARGV[0]
         th = Thread.current
@@ -1023,7 +999,7 @@ class TestRequire < Test::Unit::TestCase
 
   def test_require_with_public_method_missing
     # [Bug #19793]
-    assert_separately(["-W0", "-rtempfile"], __FILE__, __LINE__, <<~RUBY)
+    assert_separately(["-W0", "-rtempfile"], __FILE__, __LINE__, <<~RUBY, timeout: 60)
       GC.stress = true
 
       class Object
@@ -1035,18 +1011,4 @@ class TestRequire < Test::Unit::TestCase
       end
     RUBY
   end
-
-  def test_bug_21568
-    load_path = $LOAD_PATH.dup
-    loaded_featrures = $LOADED_FEATURES.dup
-
-    $LOAD_PATH.clear
-    $LOADED_FEATURES.replace(["foo.so", "a/foo.rb", "b/foo.rb"])
-
-    assert_nothing_raised(LoadError) { require "foo" }
-
-  ensure
-    $LOAD_PATH.replace(load_path) if load_path
-    $LOADED_FEATURES.replace loaded_featrures
-  end
 end
